#!/bin/sh -norc
#
# MIMEPARTS.SH 
# Author: "Garen Erdoisa" <scamper@trisk.com>
# Licence: GNU GPL
# Last Update: 8/14/2011
#
# Synopsis:
# This /bin/sh custom shell script written on behalf of the SpamBouncer project
# and as such is donated by the Author to the SpamBouncer project.
#
# This software is specifically designed to work in conjuction with and as a part of
# SpamBouncer 3.0 or higher, it is not intended for any other use.
#
# Use of this code with any other software will likely not work without major modifications.
#
# For a description of base64 encoding see:
# RFC 1521 http://www.ietf.org/rfc/rfc1521.txt Para: 5.2
#
# Expecting the following exported variables
# export SED ECHO NL DECODEBASE64 SBLOGLEVEL GREP EGREP SBDIR SPACES_TABS_TO_SPACES MESSAGETMPDIR WC SBPID DATE HOST LOGFILE SBLOGFILE SBHEADERS HASHCOMMAND
#
# Expecting the following command line arguments:
#
# These are local variables, not global. They don't belong at all in sb-config-default.rc
# and if you define them there it will have no effect.
# Don't change these or something will likely break.
thisfilename=$0
sourcefilename="$1"
OUTLINE="$2"
#PART="$3"
PART=1
RECURSELEVEL="$4"
RECURSELIMIT="$5"
LOCALBOUNDARY="$6"

# Four /bin/sh functions: splitparts(), greplocalparts(), getlinepairs(), dbecho()
# are defined by this file.
#
# The dot at the begining of the line is diliberate.
# Don't change it or it will break this script.
# The dot causes the shell_functions.sh to be read into the current
# shell environment as if it were a part of this file.
. ${SBDIR}/functions/shell_functions.sh

# dbecho is a function defined in shell_functions.sh that will only
# echo the text if the SBLOGLEVEL >= ARG1
# It's for debugging.

# Debug info
if ! [ "${OUTLINE}" = "1" ]; then
	dbecho 7 "Debug[${SBLOGLEVEL}]: [${OUTLINE}]: Recursive entry into file: ${thisfilename}"
fi
#dbecho 8 "Debug[${SBLOGLEVEL}]: MESSAGETMPDIR=\"${MESSAGETMPDIR}\""
#dbecho 8 "Debug[${SBLOGLEVEL}]: sourcefilename=${MESSAGETMPDIR}/${sourcefilename}"
#dbecho 8 "Debug[${SBLOGLEVEL}]: OUTLINE=${OUTLINE}"
#dbecho 8 "Debug[${SBLOGLEVEL}]: RECURSELEVEL=${RECURSELEVEL}"
#dbecho 8 "Debug[${SBLOGLEVEL}]: RECURSELIMIT=${RECURSELIMIT}"
#dbecho 8 "Debug[${SBLOGLEVEL}]: LOCALBOUNDARY=${LOCALBOUNDARY}"

# Unless exported from the calling process, all other variables should be local to this routine.
dbecho 8 "Debug[${SBLOGLEVEL}]: Recursion level: ${RECURSELEVEL} of ${RECURSELIMIT}; Outline Level: ${OUTLINE}"

# Outline tracks the current outline level we are at in the parse.
# dbecho 8 "Debug[${SBLOGLEVEL}]: Outline: \"${OUTLINE}\""

filename="${sourcefilename}"
LOCALPARTS=0
LOCALLINEPAIRS=0

# LOCALPARTS
dbecho 7 "Parsing: ${MESSAGETMPDIR}/${filename} LOCALBOUNDARY=${LOCALBOUNDARY}"

greplocalparts "${MESSAGETMPDIR}/${filename}" "${LOCALBOUNDARY}"
dbecho 7 "Debug[${SBLOGLEVEL}]: LOCALPARTS=${LOCALPARTS}"

VALIDATEPARTBEGIN=`${ECHO} "${LOCALPARTS}" |${SED} -e '1!d;s/^\([0-9]\{1,\}:\)\(--\)\(.\{1,\}\)$/\2/'`
VALIDATEPARTENDING=`${ECHO} "${LOCALPARTS}" |${SED} -e '$!d;s/^\([0-9]\{1,\}:--\)\(.\{1,\}\)\(--\)$/\3/'`
if [ "${VALIDATEPARTBEGIN}" = "--" ] && [ "${VALIDATEPARTENDING}" = "--" ]; then
	VALIDPART=yes
else
	VALIDPART=no
	dbecho 8 "Debug[${SBLOGLEVEL}]: Mime part was not properly closed. Attempting repair."
	if ! [ "${VALIDATEPARTENDING}" = "--" ]; then
		(${CAT} ${MESSAGETMPDIR}/${filename};${ECHO} "";${ECHO} "--${LOCALBOUNDARY}--";${ECHO} "") >>${MESSAGETMPDIR}/${filename}_repair
	fi

	greplocalparts "${MESSAGETMPDIR}/${filename}_repair" "${LOCALBOUNDARY}"
	dbecho 7 "Debug[${SBLOGLEVEL}]: repair LOCALPARTS=${LOCALPARTS}"
	VALIDATEPARTBEGIN=`${ECHO} "${LOCALPARTS}" |${SED} -e '1!d;s/^\([0-9]\{1,\}:\)\(--\)\(.\{1,\}\)$/\2/'`
	VALIDATEPARTENDING=`${ECHO} "${LOCALPARTS}" |${SED} -e '$!d;s/^\([0-9]\{1,\}:--\)\(.\{1,\}\)\(--\)$/\3/'`

	if [ "${VALIDATEPARTBEGIN}" = "--" ] && [ "${VALIDATEPARTENDING}" = "--" ]; then
		VALIDPART=yes
		dbecho 8 "Debug[${SBLOGLEVEL}]: Mime part sucessfully closed. Continuing."
		${MV} ${MESSAGETMPDIR}/${filename}_repair ${MESSAGETMPDIR}/${filename}
	else
		dbecho 8 "Debug[${SBLOGLEVEL}]: Mime part was not properly opened. Repair unsucessful, giving up."
	fi
fi

dbecho 8 "Debug[${SBLOGLEVEL}]: OUTLINE=${OUTLINE} VALIDPART=${VALIDPART}"
if [ "${VALIDPART}" = "yes" ];then
	# LOCALLINEPAIRS
	# dbecho 8 "Debug[${SBLOGLEVEL}]: getlinepairs \"${LOCALPARTS}\""
	getlinepairs "${LOCALPARTS}"
	dbecho 8 "Debug[${SBLOGLEVEL}]: LOCALLINEPAIRS=${LOCALLINEPAIRS}"
	#Commented this out because I converted the LOCALLINEPAIRS to be all on one line.
	#COUNT=`${ECHO} "${LOCALLINEPAIRS}" |grep -En "" |${SED} -nre '$s/^([0-9]+):(.+)$/\1/p'`
	#dbecho 8 "Debug[${SBLOGLEVEL}]: COUNT=$COUNT"
	for i in ${LOCALLINEPAIRS}
	do
		dbecho 8 "Debug[${SBLOGLEVEL}]: splitparts \"${MESSAGETMPDIR}/MIME_${OUTLINE}\" \"${PART}\" \"$i\""
		# This function creates the body/header files
		splitparts "${MESSAGETMPDIR}/MIME_${OUTLINE}" "${PART}" "$i"
		#Check part header to see if we need to recurse here
		dbecho 8 "Debug[${SBLOGLEVEL}]: checkpart \"${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header\""
		# some boundary tags are quoted.
		checkpart=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Mm][Uu][Ll][Tt][Ii][Pp][Aa][Rr][Tt]\/\(.\{1,\}\);\( \)\?[Bb][Oo][Uu][Nn][Dd][Aa][Rr][Yy]=".\{1,\}"\(;.*\)\?$/!d
				s/^\([C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Mm][Uu][Ll][Tt][Ii][Pp][Aa][Rr][Tt]\/.\{1,\}; \?[Bb][Oo][Uu][Nn][Dd][Aa][Rr][Yy]="\)\([^"]\{1,\}\)\("\)\(;.*\)\?$/\2/
			'`
		# some boundary tags are not quoted.
		# Note: unquoted boundary tags can not contain ; or :
		if [ "${checkpart}" = "" ]; then
		  checkpart=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Mm][Uu][Ll][Tt][Ii][Pp][Aa][Rr][Tt]\/\(.\{1,\}\);\( \)\?[Bb][Oo][Uu][Nn][Dd][Aa][Rr][Yy]=[^"]\{1,\}\(;.*\)\?$/!d
				s/^\([C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Mm][Uu][Ll][Tt][Ii][Pp][Aa][Rr][Tt]\/.\{1,\}; \?[Bb][Oo][Uu][Nn][Dd][Aa][Rr][Yy]=\)\([^";:]\{1,\}\)\(;.*\)\?$/\2/
			'`
		fi
		# Check for Content-Type headers = text or html or quoted printable
		# if such parts are found, and are base64 encoded, then decode them.
		# and/or calculate the sha1 hash of them.
		# I'm not doing anything with quoted printable or 8bit text at this time.
		#
		# Note: For future consideration, test parts that claim 7bit encoding to see if they
		# are actually 7bit. If not, the encoding tag is lying, which could be a spam indicator.
		#
		# Content-Type: text/(plain|html)
		checkcontentype_text_plain_html=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Tt][Ee][Xx][Tt]\/\([Pp][Ll][Aa][Ii][Nn]\|[Hh][Tt][Mm][Ll]\)\(;.*\)\?$/!d
			'`
		if ! [ "${checkcontentype_text_plain_html}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: checkcontentype_text_plain_html=\"${checkcontentype_text_plain_html}\""
		fi
		# Content-Type: application/(.+)
		checkcontentype_application=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Aa][Pp][Pp][Ll][Ii][Cc][Aa][Tt][Ii][Oo][Nn]\/.\{1,\}\(;.*\)\?$/!d
			'`
		if ! [ "${checkcontentype_application}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: checkcontentype_application=\"${checkcontentype_application}\""
		fi
		# Content-Type: image/(.+)
		checkcontentype_image=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: [Ii][Mm][Aa][Gg][Ee]\/.\{1,\}\(;.*\)\?$/!d
			'`
		if ! [ "${checkcontentype_image}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: checkcontentype_image=\"${checkcontentype_image}\""
		fi

		# Content-Transfer-Encoding: base64	
		checkencodingbase64=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Rr][Aa][Nn][Ss][Ff][Ee][Rr]-[Ee][Nn][Cc][Oo][Dd][Ii][Nn][Gg]: [Bb][Aa][Ss][Ee]64\(;.*\)\?$/!d
			'`
		if ! [ "${checkencodingbase64}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: Found base64 encoding tag in MIME_${OUTLINE}_${PART}.header"
			checkcontent_disposition_attachment_filename=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
				${SED} -e '/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Dd][Ii][Ss][Pp][Oo][Ss][Ii][Tt][Ii][Oo][Nn]: [Aa][Tt][Tt][Aa][Cc][Hh][Mm][Ee][Nn][Tt].*[; ]\?[Ff][Ii][Ll][Ee][Nn][Aa][Mm][Ee]=.\{1,\}\(;.*\)\?$/!d' |\
				${SED} -e 's/^\([C][Oo][Nn][Tt][Ee][Nn][Tt]-[Dd][Ii][Ss][Pp][Oo][Ss][Ii][Tt][Ii][Oo][Nn]: [Aa][Tt][Tt][Aa][Cc][Hh][Mm][Ee][Nn][Tt].*[; ]\?[Ff][Ii][Ll][Ee][Nn][Aa][Mm][Ee]=\)\(.\{1,\}\)\(\(;.*\)\?\)$/\2/' |\
				${SED} -e 's/^\("\)\(.\{1,\}\)\("\)$/\2/'
			`
			if ! [ "${checkcontent_disposition_attachment_filename}" = "" ];then
				dbecho 8 "Debug[${SBLOGLEVEL}]: checkcontent_disposition_attachment_filename=\"${checkcontent_disposition_attachment_filename}\""
			fi
			checkcontent_type_name=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
				${SED} -e '/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: .*[; ]\?[Nn][Aa][Mm][Ee]=.\{1,\}\(;.*\)\?$/!d' |\
				${SED} -e 's/^\([C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Yy][Pp][Ee]: .*[; ]\?[Nn][Aa][Mm][Ee]=\)\(.\{1,\}\)\(\(;.*\)\?\)$/\2/' |\
				${SED} -e 's/^\("\)\(.\{1,\}\)\("\)$/\2/'
			`
			if ! [ "${checkcontent_type_name}" = "" ];then
				dbecho 8 "Debug[${SBLOGLEVEL}]: checkcontent_type_name=\"${checkcontent_type_name}\""
			fi
		fi

		# Content-Transfer-Encoding: 7bit
		checkencoding7bit=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header |\
			${SED} -e '
				/^[C][Oo][Nn][Tt][Ee][Nn][Tt]-[Tt][Rr][Aa][Nn][Ss][Ff][Ee][Rr]-[Ee][Nn][Cc][Oo][Dd][Ii][Nn][Gg]: \?.*7[Bb][Ii][Tt]\(;.*\)\?$/!d
			'`
		if ! [ "${checkencoding7bit}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: Found 7bit encoding tag in MIME_${OUTLINE}_${PART}.header"
			# dbecho 8 "Debug[${SBLOGLEVEL}]: ${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |${TR} -d \"\000-\177\" |${WC} -c"
			checkcontent7bit=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |${TR} -d "\000-\177" |${WC} -c`
			if ! [ "${checkcontent7bit}" = "0" ]; then
				dbecho 8 "Debug[${SBLOGLEVEL}]: 8bit byte count=${checkcontent7bit} MIME_${OUTLINE}_${PART}.body is not 7bit!"
				# This code mimics the following for SpamBouncer logging:
				#  SBLOG="C3N: MIME Part ${OUTLINE}_${PART} claims 7bit encoding but has 8bit content"
				#  INCLUDERC=${SBDIR}/functions/loglevel.rc
				# Note the header in the log
				dbecho 3 "SBNote: MIME Part ${OUTLINE}_${PART} claims 7bit encoding but has 8bit content"
				if [ "${SBHEADERS}" = "COMPLETE" ] || [ "${SBHEADERS}" = "BRIEF" ]; then
					# Add the header to the email.
					${ECHO} "X-SBNote: MIME Part ${OUTLINE}_${PART} claims 7bit encoding but has 8bit content" >>${MESSAGETMPDIR}/sbheaders-created-during-autopsy
				fi
			else
				dbecho 8 "Debug[${SBLOGLEVEL}]: Confirmed: 8bit byte count=${checkcontent7bit} MIME_${OUTLINE}_${PART}.body is 7bit."
			fi
		fi

		# This is where we decide what to include in the Message Body Autopsy. The rest will be ignored.

		# Pick up the part header files if any and include them in the autopsy so
		# SpamBouncer can check the language charset headers, and other Content- headers.
		if [ -f ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header ]; then
			dbecho 7 "Debug[${SBLOGLEVEL}]: Including file: ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header"
			${ECHO} "${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.header" >>${MESSAGETMPDIR}/plain_body_files_to_parse.txt
		fi
		# If the Content-Type: mime part header claims the part is text/html, then include it in the autopsy. Decode base64 if necessary.
		if ! [ "${checkcontentype_text_plain_html}" = "" ]; then
			if ! [ "${checkencodingbase64}" = "" ]; then
				#dbecho 8 "Debug[${SBLOGLEVEL}]: ${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |${DECODEBASE64} >${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain"
				${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |\
				${DECODEBASE64} |\
				${SED} -e 's/^From /\\From /;s/^\.$/\\./' >${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain
				dbecho 7 "Debug[${SBLOGLEVEL}]: Including file: ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain"
				${ECHO} "${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain" >>${MESSAGETMPDIR}/plain_body_files_to_parse.txt
			else
				#dbecho 8 "Debug[${SBLOGLEVEL}]: ${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body >${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain"
				${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |\
				${SED} -e 's/^From /\\From /;s/^\.$/\\./' >${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain
				dbecho 7 "Debug[${SBLOGLEVEL}]: Including file: ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain"
				${ECHO} "${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.plain" >>${MESSAGETMPDIR}/plain_body_files_to_parse.txt
			fi
		fi
		# If the Content-Type: mime part header claims the part is an application or image then
		# Calculate the sha1 hash of the MIME body part or if the part is base64 encoded,
		# calculate the sha1 hash of the decoded body part.
		# This hash could be checked against or added to a database of spammy sha1 hash's.
		# or against a databse of sha1 hashes that have been whitelisted.
		# such as might exist for bank logo .jpg's, trademark loogo's etc.
		# ie: ~/.sbSpammyHashCache
		if ! [ "${checkcontentype_application}" = "" ] || \
		   ! [ "${checkcontentype_image}" = "" ] || \
		   ! [ "${checkcontent_disposition_attachment_filename}" = "" ] || \
		   ! [ "${checkcontent_type_name}" = "" ];\
		then
			if ! [ "${checkencodingbase64}" = "" ]; then
				(((${ECHO} "base64 decoded sha1 hash=\"<";\
				   ${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |\
				   ${DECODEBASE64} |\
				   ${HASHCOMMAND};\
				   ${ECHO} ">\"")|\
				  ${TR} -d '\012\015');\
				 ${ECHO} "") \
					>${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash
				dbecho 7 "Debug[${SBLOGLEVEL}]: Including file: ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash"
				${ECHO} "${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash" \
					>>${MESSAGETMPDIR}/plain_body_files_to_parse.txt
			else
				(((${ECHO} "sha1 hash=\"<";\
				   ${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body |\
				   ${HASHCOMMAND};\
				   ${ECHO} ">\"")|\
				  ${TR} -d '\012\015');\
				 ${ECHO} "") \
					>${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash
				dbecho 7 "Debug[${SBLOGLEVEL}]: Including file: ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash"
				${ECHO} "${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash" \
					>>${MESSAGETMPDIR}/plain_body_files_to_parse.txt
			fi
			hash=`${CAT} ${MESSAGETMPDIR}/MIME_${OUTLINE}_${PART}.body.hash |\
				${SED} -re 's/^(.*sha1 hash=")([<].+[>])"$/\2/'`
			dbecho 8 "Debug[${SBLOGLEVEL}]: hash=\"${hash}\""
			check_spammy_mimepart_hash=''
			if [ -f ${HOME}/.sbSpammyHashCache ]; then
				# dbecho 8 "Debug[${SBLOGLEVEL}]: ${GREP} -x \"${hash}\" ${HOME}/.sbSpammyHashCache"
				#
				# Note: this code could be adjusted to check a dns blocklist for a spammy hash instead of a file
				# ie: something like this:
				# check_spammy_mimepart_hash=`${HOST} ${hash}.hashcheck.dnsbl.example.org`
				# where 127.0.0.2 = hash signature of a mime part found in spamtraps
				#       127.0.0.3 = hash signature of a known virus/trojen or other dangerous attachment
				#	127.0.0.4 = hash signature of a known good logo/trademark, possible spam victim.
				#
				# Check a file of sha1 signatures of mime parts found in spam.
				# special case:
				# Note: da39a3ee5e6b4b0d3255bfef95601890afd80709 is the sha1 hash of a zero byte or NULL message
				# ie: echo -ne "" |openssl base64 -d |openssl sha1
				# if that is the result of the decode, then ignore it.
				# The hash is likely the result of a previously defanged message.
				# probably from a virus filter that removed the part, but otherwise 
				# left the mime structure intact then delivered the defanged message.
				if ! [ "${hash}" = "<da39a3ee5e6b4b0d3255bfef95601890afd80709>" ]; then
					check_spammy_mimepart_hash=`${GREP} -x "${hash}" ${HOME}/.sbSpammyHashCache`
				fi
				# dbecho 8 "Debug[${SBLOGLEVEL}]: check_spammy_mimepart_hash=\"${check_spammy_mimepart_hash}\""
			fi
			if ! [ "${check_spammy_mimepart_hash}" = "" ]; then
				# This code mimics the following for SpamBouncer logging:
				#  SBLOG="C3T: Spammy Hash (MIME part ${OUTLINE}_${PART})"
				#  INCLUDERC=${SBDIR}/functions/loglevel.rc
				dbecho 4 "Found sha1 hash \"${hash}\" cached."
				dbecho 3 "SBPattern: Spammy Hash (MIME part ${OUTLINE}_${PART})"
				if [ "${SBHEADERS}" = "COMPLETE" ] || [ "${SBHEADERS}" = "BRIEF" ]; then
					${ECHO} "X-SBPattern: Spammy Hash (MIME part ${OUTLINE}_${PART})" >>${MESSAGETMPDIR}/sbheaders-created-during-autopsy
				fi
			fi
		fi

		# Are we done extracting mime parts? If not then recurse to extract the next set of MIME parts in outline format.
		# This is where this file calls itself to do this magic.
		if ! [ "${checkpart}" = "" ]; then
			dbecho 8 "Debug[${SBLOGLEVEL}]: checkpart=\"${checkpart}\""
			if ! [ "${RECURSELEVEL}" -ge "${RECURSELIMIT}" ]; then
				# dbecho 8 "Debug[${SBLOGLEVEL}]: ${SBDIR}/functions/mimeparts.sh MIME_${OUTLINE}_${PART} ${OUTLINE}_${PART} $[RECURSELEVEL+1] ${RECURSELIMIT} ${checkpart}"
				# This is where we do the recursion if it's needed.	
				${SBDIR}/functions/mimeparts.sh "MIME_${OUTLINE}_${PART}" "${OUTLINE}_${PART}" "${PART}" "$[RECURSELEVEL+1]" "${RECURSELIMIT}" "${checkpart}"
				dbecho 7 "Debug[${SBLOGLEVEL}]: [${OUTLINE}_${PART}] Recursive exit from file: ${thisfilename}"
			else
				dbecho 3 "MIME Recurse limit reached: RECURSELEVEL=${RECURSELEVEL} >= RECURSELIMIT=${RECURSELIMIT}"
			fi
		fi
		PART=$[PART+1]
	done
fi
